using System;
using System.Collections.Generic;
using System.Text;

namespace LocalTimeReader
{
    public class Message
    {
        public enum MessageStatus
        {
            Complete,
            InProgress,
            ResendError,
            IgnoreError
        }

        private byte code;
        private MessageStatus status;
        private long localTime;
        private long remoteTime;
        private byte index;
        private byte[] timeBytes = new byte[16];
        private int dataCount;

        public Message(byte first)
        {
            code = first;
            status = MessageStatus.InProgress;

            if (code == 0x01 || code == 0x02 || code == 0x03)
                status = MessageStatus.Complete;
            else
                dataCount = 0;
        }

        public void Add(byte next)
        {
            if (code != 0x04 && code != 0x05)
                return;

            if (dataCount == 0)
            {
                index = next;
                dataCount = 1;
                return;
            }

            timeBytes[dataCount-1] = next;
            dataCount++;

            if (dataCount == 9 && code == 0x04)
            {
                status = MessageStatus.Complete;
                localTime = (long)BitConverter.ToInt64(timeBytes, 0);
            }

            if (dataCount == 17 && code == 0x05)
            {
                status = MessageStatus.Complete;
                localTime = (long)BitConverter.ToInt64(timeBytes, 0);
                remoteTime = (long)BitConverter.ToInt64(timeBytes, 8);
            }
        }

        public byte Code
        {
            get { return code; }
        }

        public MessageStatus Status
        {
            get { return status; }
        }

        public long LocalTime
        {
            get { return localTime; }
        }

        public long RemoteTime
        {
            get { return remoteTime; }
        }

        public byte Index
        {
            get { return index; }
        }
    }
}
