using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace LocalTimeReader
{
    class ThreadPage: TabPage
    {
        private System.Windows.Forms.ListView listView;
        private System.Windows.Forms.ColumnHeader localTimeAHeader;
        private System.Windows.Forms.ColumnHeader remoteTimeHeader;
        private System.Windows.Forms.ColumnHeader numberHeader;
        private System.Windows.Forms.ColumnHeader localTimeBHeader;

        private LocalTimeReaderPort localTimeReaderPort = null;
        private int tickCount;
        private int tickTotal;
        private byte index;
        private Timer intervalTimer = new Timer();
        private MainForm parent;
        private LocalTimeReaderPort readerPort;

        public ThreadPage(byte idx, MainForm prt)
        {
            index = idx;
            parent = prt;

            this.listView = new System.Windows.Forms.ListView();
            this.numberHeader = new System.Windows.Forms.ColumnHeader();
            this.localTimeAHeader = new System.Windows.Forms.ColumnHeader();
            this.remoteTimeHeader = new System.Windows.Forms.ColumnHeader();
            this.localTimeBHeader = new System.Windows.Forms.ColumnHeader();
            // 
            // listView
            // 
            this.listView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.numberHeader,
            this.localTimeAHeader,
            this.remoteTimeHeader,
            this.localTimeBHeader});
            this.listView.FullRowSelect = true;
            this.listView.GridLines = true;
            this.listView.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
            this.listView.Location = new System.Drawing.Point(0, 0);
            this.listView.MultiSelect = false;
            this.listView.Name = "listView";
            this.listView.Size = new System.Drawing.Size(423, 416);
            this.listView.TabIndex = 0;
            this.listView.UseCompatibleStateImageBehavior = false;
            this.listView.View = System.Windows.Forms.View.Details;
            // 
            // numberHeader
            // 
            this.numberHeader.Text = "No";
            this.numberHeader.Width = 30;
            // 
            // localTimeAHeader
            // 
            this.localTimeAHeader.Text = "Local Time A";
            this.localTimeAHeader.Width = 120;
            // 
            // remoteTimeHeader
            // 
            this.remoteTimeHeader.Text = "Remote Time";
            this.remoteTimeHeader.Width = 120;
            // 
            // localTimeBHeader
            // 
            this.localTimeBHeader.Text = "Local Time B";
            this.localTimeBHeader.Width = 120;


            this.Controls.Add(this.listView);
            this.Name = "threadTab" + index.ToString();
            this.Text = "Thread " + index.ToString();
            this.Location = new System.Drawing.Point(0, 0);
            this.Padding = new System.Windows.Forms.Padding(3);
            this.Size = new System.Drawing.Size(432, 422);
            this.TabIndex = 0;

            intervalTimer.Tick += new EventHandler(intervalTimer_Tick);
            intervalTimer.Enabled = false;
        }

        void intervalTimer_Tick(object sender, EventArgs e)
        {
            if (localTimeReaderPort != null)
                localTimeReaderPort.SendTimeRequest(index, DateTime.Now.Ticks);

            if (tickCount == tickTotal)
            {
                intervalTimer.Enabled = false;
                parent.RunStop();
            }

            tickCount++;

        }

        private string GetTimeString(DateTime time)
        {
            return time.Hour.ToString() + ":"
                + time.Minute.ToString() + ":"
                + time.Second.ToString() + "."
                + time.Millisecond.ToString();
        }

        public void ReceivedTime(long localTime, long remoteTime)
        {
            ListViewItem item = listView.Items.Add((listView.Items.Count+1).ToString());

            if (localTime < DateTime.MinValue.Ticks || localTime > DateTime.MaxValue.Ticks)
                item.SubItems.Add("Invalid Time");
            else
                item.SubItems.Add(GetTimeString(new DateTime(localTime)));
            if (remoteTime < DateTime.MinValue.Ticks || remoteTime > DateTime.MaxValue.Ticks )
                item.SubItems.Add("Invalid Time");
            else
                item.SubItems.Add(GetTimeString(new DateTime(remoteTime)));
            item.SubItems.Add(GetTimeString(DateTime.Now));

        }

        public void Run(LocalTimeReaderPort port, int interval, int noTicks)
        {
            localTimeReaderPort = port;
            tickTotal = noTicks;

            intervalTimer.Interval = interval;

            listView.Items.Clear();
            tickCount = 0;

            localTimeReaderPort.SendTimeRequest(index, DateTime.Now.Ticks);

            if (tickTotal > 1)
                intervalTimer.Enabled = true;

        }


    }
}
