// RemoteTimeReaderC.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "conio.h"
#include "windows.h"
#include "winbase.h"

enum
{
    NotConnected,
    ConnectionEstablished,
    ConnectionIdle,
    CheckConnected
}status;


bool SendInfoMessage(HANDLE hSerialPort, char* data)
{
	char message[18];

	message[0] = 0x05;
	memcpy(&message[1], &data[1], 9);

	SYSTEMTIME st;

	union
	{
		FILETIME ft;
		char bytes[8];
	}nowTime;

	GetLocalTime(&st);
	SystemTimeToFileTime(&st, &(nowTime.ft));

	memcpy(&message[10], nowTime.bytes, 8);

	DWORD bytesSent;

	if (WriteFile(hSerialPort, message, 18, &bytesSent, NULL) == 0)
	{
		cprintf("Failed to send message\n");
		return false;
	}

	if (bytesSent != 18)
	{
		cprintf("Failed to send complete message\n");
		return false;
	}

	return true;
}

bool SendOneByteMessage(HANDLE hSerialPort, char dataByte)
{
		DWORD bytesSent;

		if (WriteFile(hSerialPort, &dataByte, 1, &bytesSent, NULL) == 0)
		{
			cprintf("Failed to send message\n");
			return false;
		}

		if (bytesSent != 1)
		{
			cprintf("Failed to send complete message\n");
			return false;
		}

		return true;
}

bool AcknowledgementTimerElapsed(HANDLE hSerialPort)
{
	/*
	if (status == NotConnected)
		cprintf("Ack Time NotConnected\n");
	else if (status == ConnectionEstablished)
		cprintf("Ack Time ConnectionEstablished\n");
	else if (status ==	ConnectionIdle)
		cprintf("Ack Time ConnectionIdle\n");
	else if (status == CheckConnected)
		cprintf("Ack Time CheckConnected\n");
	else
		cprintf("Ack Time Invalid\n");
	*/	

    if (status == ConnectionEstablished)
        status = ConnectionIdle;
    else if (status == ConnectionIdle || status == NotConnected)
    {
        if (status == ConnectionIdle)
            status = CheckConnected;

        return SendOneByteMessage(hSerialPort, 0x01);
    }
    else if (status == CheckConnected)
    {
        cprintf("Communication lost - Timeout on acknowledge request\n");
        return false;
    }

    return true;
}

int _tmain(int argc, _TCHAR* argv[])
{
	char receivedMessage[10];
	int messageIndex = 0;
	status = NotConnected;

	HANDLE hSerialPort = CreateFile(_T("COM1"), 
            GENERIC_READ | GENERIC_WRITE, 
            0, 
            NULL, 
            OPEN_EXISTING, 
            0, 
            NULL);

	if (hSerialPort == INVALID_HANDLE_VALUE)
	{
		cprintf("Cannot get serial port handle\n");
		return 0;
	}

	DCB dcbConfig;

	if(GetCommState(hSerialPort, &dcbConfig))
	{
		dcbConfig.BaudRate = 9600;
		dcbConfig.ByteSize = 8;
		dcbConfig.Parity = NOPARITY;
		dcbConfig.StopBits = ONESTOPBIT;
		dcbConfig.fDsrSensitivity = FALSE;
		dcbConfig.fDtrControl = DTR_CONTROL_DISABLE;
		dcbConfig.fBinary = TRUE;
		dcbConfig.fRtsControl = RTS_CONTROL_ENABLE;
		dcbConfig.fOutxCtsFlow = TRUE;
	}
	else
	{
		cprintf("Cannot get comm state\n");
		return 0;
	}

	if(!SetCommState(hSerialPort, &dcbConfig))
	{
		cprintf("Cannot set comm state\n");
		return 0;
	}

	COMMTIMEOUTS commTimeout;

	if(GetCommTimeouts(hSerialPort, &commTimeout)) /* Configuring Read & Write Time Outs */
  	{
  		commTimeout.ReadIntervalTimeout = 1000;
  		commTimeout.ReadTotalTimeoutConstant = 1000;
  		commTimeout.ReadTotalTimeoutMultiplier = 0;
  		commTimeout.WriteTotalTimeoutConstant = 10000;
  		commTimeout.WriteTotalTimeoutMultiplier = 0;
  	} 
  	else
	{
		cprintf("Get timeout fail\n");
		return 0;
	}
  
  	if(!SetCommTimeouts(hSerialPort, &commTimeout))
	{
		cprintf("Set timeout fail\n");
		return 0;
	}
   
	while(true)
	{
		char buffer[1000];
		DWORD readSize;

		if (ReadFile(hSerialPort, buffer, 1000, &readSize, NULL) != 0)
		{
			for (int iii=0; iii<1000 && iii<readSize; iii++)
			{
				if (status != NotConnected)
					status = ConnectionEstablished;

				if (messageIndex == 0)
				{
					if (buffer[iii] == 0x02)
					{
						status = ConnectionEstablished;
						if (!SendOneByteMessage(hSerialPort, 0x03))
							return 0;
					}

					if (status == NotConnected)
						continue;

					if (buffer[iii] == 0x04)
					{
						receivedMessage[0] = buffer[iii];
						messageIndex++;
					}
				}
				else
				{
					receivedMessage[messageIndex] = buffer[iii];
					messageIndex++;
					if (messageIndex == 10)
					{
						messageIndex = 0;
						if (!SendInfoMessage(hSerialPort, receivedMessage))
							return 0;
					}
				}
			}
		}

		if (!AcknowledgementTimerElapsed(hSerialPort))
			break;
 	}

	return 0;
}


