using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;
using System.Threading;
using LocalTimeReader;

namespace RemoteTimeReader
{
    class Program
    {
        public enum PortStatus
        {
            NotConnected,
            ConnectionEstablished,
            ConnectionIdle,
            CheckConnected
        }

        static private bool newMessage;
        static private Message receivedMessage;
        static private PortStatus status;
        static private SerialPort port = new SerialPort();

        static void Main(string[] args)
        {
            newMessage = true;
            status = PortStatus.NotConnected;

            if (args.Length == 0)
                port.PortName = "COM1";
            else
                port.PortName = args[0];

            port.BaudRate = 9600;
            port.Parity = Parity.None;
            port.DataBits = 8;
            port.StopBits = StopBits.One;
            port.Handshake = Handshake.RequestToSend;
            port.ReadTimeout = 500;
            port.WriteTimeout = 500;

            try
            {
                port.Open();
                // Get rid of crap build up particularly if devices
                // are switched on first
                port.DiscardInBuffer();
            }
            catch (Exception)
            {
                Console.WriteLine("Failed to open serial port");
                return;
            }

            int sleepCount = 0;

            while (port.CtsHolding)
            {
                int totalBytes = port.BytesToRead;

                if (totalBytes == 0)
                {
                    Thread.Sleep(10);
                    if (sleepCount == 100)
                    {
                        if (!acknowledgementTimerElapsed())
                            return;
                        sleepCount = 0;
                    }
                        
                    sleepCount++;
                    continue;
                }

                int byteCount = 0;

                while (byteCount < totalBytes)
                {
                    if (newMessage)
                    {
                        if (status != PortStatus.NotConnected)
                            status = PortStatus.ConnectionEstablished;

                        receivedMessage = new Message((byte)port.ReadByte());

                        if (receivedMessage.Status == Message.MessageStatus.InProgress)
                            newMessage = false;
                        else
                            ReceivedMessage(receivedMessage);
                    }
                    else
                    {
                        receivedMessage.Add((byte)port.ReadByte());
                        if (receivedMessage.Status != Message.MessageStatus.InProgress)
                        {
                            newMessage = true;
                            ReceivedMessage(receivedMessage);
                        }
                    }

                    byteCount++;
                }
            }

            Console.WriteLine("Serial port connection is down");
        }

        static private bool acknowledgementTimerElapsed()
        {
            if (status == PortStatus.ConnectionEstablished)
                status = PortStatus.ConnectionIdle;
            else if (status == PortStatus.ConnectionIdle || status == PortStatus.NotConnected)
            {
                if (status == PortStatus.ConnectionIdle)
                    status = PortStatus.CheckConnected;
                byte[] dataBytes = new byte[1];
                dataBytes[0] = 0x01;
                port.Write(dataBytes, 0, dataBytes.Length);
            }
            else if (status == PortStatus.CheckConnected)
            {
                Console.WriteLine("Communication lost - Timeout on acknowledge request");
                return false;
            }

            return true;
        }

        static private void ReceivedMessage(Message receivedMessage)
        {
            if (receivedMessage.Code == 0x02)
            {
                status = PortStatus.ConnectionEstablished;
                byte[] dataBytes = new byte[1];
                dataBytes[0] = 0x03;
                port.Write(dataBytes, 0, dataBytes.Length);
            }

            if (status == PortStatus.NotConnected)
                return;

            if (receivedMessage.Code == 0x04)
            {
                byte[] dataBytes = new byte[18];
                dataBytes[0] = 0x05;
                dataBytes[1] = receivedMessage.Index;
                (BitConverter.GetBytes(receivedMessage.LocalTime)).CopyTo(dataBytes, 2);
                (BitConverter.GetBytes(DateTime.Now.Ticks)).CopyTo(dataBytes, 10);

                port.Write(dataBytes, 0, dataBytes.Length);
            }
        }

    }
}
